var gulp = require('gulp'),
elixir = require('laravel-elixir'),
config = elixir.config,
inSequence = require('run-sequence'),
utilities = require('laravel-elixir/ingredients/commands/Utilities'),
notifications = require('laravel-elixir/ingredients/commands/Notification'),
gulpIf = require('gulp-if'),
sourcemaps = require('gulp-sourcemaps'),
uglify = require('gulp-uglify'),
rename = require('gulp-rename'),
source = require('vinyl-source-stream'),
buffer = require('vinyl-buffer'),
browserify = require('browserify'),
watchify = require('watchify'),
_  = require('underscore'),
path = require('path'),
babelify = require('babelify');

var createBundle = function(watch) {
  var stream;
  var notification = new notifications();
  var onError = function(e) {
    notification.error(e, 'Watchify Compilation Failed!');
    this.emit('end');
  };

  var bundle = function(b, options) {
    return b.bundle()
    .on('error', onError)
    .pipe(source(path.basename(options.rename)))
    .pipe(buffer())
    .pipe(gulpIf(options.debug, sourcemaps.init({ loadMaps: true })))
    .pipe(gulpIf(!options.debug, uglify()))
    .pipe(gulpIf(options.debug, sourcemaps.write(".")))
    .pipe(gulp.dest(options.output));
  };

  config.toWatchify.forEach(function(options) {
    var b = browserify(options);
    _.forEach(options.external, function (external) {
      if (!_.isArray(external)) external = [external, {}];
      var file = external[0];
      var options = external[1];
      // console.log(file, options);
      b.require(file, options);
    });

    if (watch) {
      b = watchify(b);

      b.on('update', function() {
        utilities.logTask('Running Watchify', options.entries);
        bundle(b, options);
      });
    }

    utilities.logTask('Running Watchify', options.entries);
    stream = bundle(b, options);
  });

  return stream;
};

/**
* Create the Gulp task.
*
* @return {void}
*/
var buildTask = function() {
  gulp.task('watchify', function() {
    return createBundle(false);
  });
};

/**
* Create elixir extension
*/
elixir.extend('watchify', function (options) {
  if (!_.isArray(config.toWatchify)) {
    config.toWatchify = [];
  }

  options = _.extend({
    debug:         ! config.production,
    rename:        null,
    srcDir:        config.assetsDir + 'js',
    output:        config.jsOutput,
    transform:     [ [ babelify, { stage: 0 }] ]
  }, options);

  config.toWatchify.push(options);

  buildTask();

  // Use null instead of browserify so gulp doesn't
  // compile each bundle twice on start.
  this.registerWatcher(null, function() {
    return createBundle(true);
  });

  return this.queueTask('watchify');
});

module.exports = elixir;
